// objekat.h - Apstraktna klasa objekata.

#ifndef _objekat_h_
#define _objekat_h_

#include <iostream>
#include <typeinfo>
using namespace std;

class Obj {
  static int posId;      // Poslednji identifikator.
  const int id;          // Identifikator objekta.
  virtual bool jednako(const Obj& obj) const =0;       // Da li su jednaki?
  virtual void pisi (ostream& it) const =0;            // Pisanje.
  virtual void citaj(istream& ut) =0;                  // Citanje.
public:
  Obj(): id(++posId) {}           // Nov objekat dobija nov identifikator.
  Obj(const Obj&): id(++posId) {} // Kopija dobija nov identifikator.
  Obj& operator=(const Obj&)      // Levom operandu se ne menja identifik.
    { return *this; };
  virtual ~Obj() {}                                         // Unitavanje.
  const char* klasa() const                                 // Ime klase.
    { return typeid(*this).name(); }
  virtual Obj* kopija() const =0;                           // Kopiranje.
  friend bool operator==(const Obj& o1, const Obj& o2)      // o1 == o2
    { return typeid(o1)==typeid(o2) && o1.jednako(o2); }
  friend bool operator!=(const Obj& o1, const Obj& o2)      // o1 != o2
    { return !(o1 == o2); }
  friend ostream& operator<<(ostream& it, const Obj& obj)   // it << obj
    { it << obj.klasa() << obj.id; obj.pisi(it); return it; }
  friend istream& operator>>(istream& ut, Obj& obj)         // ut >> obj
    { obj.citaj(ut); return ut; }
};

#endif
