// racmatr.C - Metode i funkcije uz klasu matrica racionalnih brojeva.

#include "racmatr.h"
#include <iomanip>
using namespace std;

namespace Aritmetika {
  void RacMatr::pravi(int v, int k, CRB& r) {    // Dodela memorije.
    m = v; n = k;
    niz = new RacBroj* [m];
    for (int i=0; i<m; i++) {
      niz[i] = new RacBroj [n];
      for (int j=0; j<n; niz[i][j++]=r);
    }
  }

  void RacMatr::kopiraj(CRM& a) {                // Kopiranje.
    m = a.m; n = a.n;
    niz = new RacBroj* [m];
    for (int i=0; i<m; i++) {
      niz[i] = new RacBroj [n];
      for (int j=0; j<n; j++) niz[i][j] = a.niz[i][j];
    }
  }

  void RacMatr::brisi() {                        // Oslobadanje memorije.
    for (int i=0; i<m; delete [] niz[i++]);
    delete [] niz; niz = 0; m = n = 0;
  }

  RacMatr RacMatr::operator-() const {           // -A
    RacMatr a(*this);
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
        a.niz[i][j] = -a.niz[i][j];
    return a;
  }

  RacMatr T(const RacMatr& a) {                  // Transponovana matrica.
    RacMatr b(a.n, a.m);
    for (int i=0; i<a.m; i++)
      for (int j=0; j<a.n; j++) b.niz[j][i] = a.niz[i][j];
    return b;
  }

  RacMatr RacMatr::inv_det(RacBroj& det) const { // Inverzna matrica i
    if (m != n) throw GRacMatKvd();              //   determinata.
    RacMatr b(m, 2*m); RacBroj d(1);
    // Stvaranje proirene matrice:
    for (int i=0; i<m; i++)
      for (int j=0; j<m; j++)
        { b.niz[i][j] = niz[i][j]; b.niz[i][j+m] = i==j; }
    // Eliminacija unapred:
    for (int i1=0; i1<m; i1++) {
      // obezbedivanje nenulte vrednosti na dijagonali:
      if (b.niz[i1][i1] == 0) {
        int j=i1+1; while (j<n && b.niz[j][i1]==0) j++;
        if (j == n) throw GRacMatInv();
        RacBroj* p = b.niz[i1]; b.niz[i1] = b.niz[j]; b.niz[j] = p;
      }
      // normiranje vrste:
      if (b.niz[i1][i1] != 1) {
        d *= b.niz[i1][i1];
        for (int j=2*m-1; j>=i1; j--) b.niz[i1][j] /= b.niz[i1][i1];
      }
      // stvaranje nula u koloni i1 nadole:
      for (int i2=i1+1; i2<m; i2++)
        if (b.niz[i2][i1] != 0)
          for (int j=2*m-1; j>=i1; j--)
            b.niz[i2][j] -= b.niz[i1][j] * b.niz[i2][i1];
    }
    // Zamenjivanje unazad (stvaranje nula u kolonama nagore):
    for (int i1=m-1; i1>0; i1--) {
      for (int i2=i1-1; i2>=0; i2--)
        if (b.niz[i2][i1] != 0)
          for (int j=2*m-1; j>=i1; j--)
            b.niz[i2][j] -= b.niz[i1][j] * b.niz[i2][i1];
    }
    // Vracanje rezultata:
    RacMatr a(m, m);
    for (int i=0; i<m; i++)
      for (int j=0; j<m; j++) a.niz[i][j] = b.niz[i][j+m];
    det = d;
    return a;
  }
 
  RacMatr& RacMatr::operator+=(CRM& b) {               // A += B
    if (m!=b.m || n!=b.n) throw GRacMatDim();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++) niz[i][j] += b.niz[i][j];
    return *this;
  }

  RacMatr& RacMatr::operator-=(CRM& b) {               // A -= B
    if (m!=b.m || n!=b.n) throw GRacMatDim();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++) niz[i][j] -= b.niz[i][j];
    return *this;
  }

  RacMatr RacMatr::operator*(CRM& b) const {           // A *  B
    if (n != b.m) throw GRacMatDim();
    RacMatr c(m, b.n);
    for (int i=0; i<m; i++)
      for (int k=0; k<b.n; k++)
        for (int j=0; j<n; j++)
          c.niz[i][k] += niz[i][j] * b.niz[j][k];
    return c;
  }

  RacMatr& RacMatr::operator+=(CRB& r) {               // A += r
    for (int i=0; i<m; i++)
      for (int j=0; j<n; niz[i][j++]+=r);
    return *this;
  }

  RacMatr& RacMatr::operator*=(CRB& r) {               // A *= r
    for (int i=0; i<m; i++)
      for (int j=0; j<n; niz[i][j++]*=r);
    return *this;
  }

  bool RacMatr::operator==(CRM& b) const {             // A == B
    if (m!=b.m || n!=b.n) return false;
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
        if (niz[i][j] != b.niz[i][j]) return false;
    return true;
  }

  istream& operator>>(istream& ut, RacMatr& a) {       // Citanje.
    for (int i=0; i<a.m; i++)
      for (int j=0; j<a.n; ut>>a.niz[i][j++]);
    return ut;
  }

  ostream& operator<<(ostream& it, const RacMatr& a) { // Pisanje.
    int sir = it.width(1);
    for (int i=0; i<a.m; i++) {
      it << '[';
      for (int j=0; j<a.n; it << setw(sir) << a.niz[i][j++]);
      it << ']' << endl;
    }
    return it;
  }
}
